local name, addon_table = ...

local _G = _G
local RW = _G.RW3
local expect = RW.Debug.expect
local DEBUG = RW.DEBUG
local Objects = RW.Objects
local Media = RW.Media

local plugin = addon_table.Plugin
local Constants = plugin.Constants
Constants:Register("ICONSIDE_LEFT", 101)
Constants:Register("ICONSIDE_RIGHT", 102)

local OnDragStart
local OnDragStop
local OnUpdate
local Anchor = {}
local Counter = 0

function plugin:CreateAnchor(name)
	local anchor = setmetatable({
		Frame = Objects:GetAnchor(UIParent),
		Name = name,
		Bars = {},
	}, {__index = Anchor})
	
	anchor.Frame.Text:SetFormattedText("Timer: %s", name)
	anchor.Frame:RegisterForDrag("LeftButtoN")
	anchor.Frame:SetScript("OnDragStart", OnDragStart)
	anchor.Frame:SetScript("OnDragStop", OnDragStop)
	anchor.Frame.obj = anchor
	
	return anchor
end

function Anchor:ApplySettings()
	for i, bar in self:IterateBars() do
		bar.frame:SetWidth(self.db.Width)
		bar.frame:SetHeight(self.db.Height)
		bar:ApplySettings()
	end
	
	self:DoLayout()
end

function Anchor:Delete()
	for i, b in self:IterateBars() do
		b:Delete()
	end
	wipe(self.Bars)
	self.Frame = self.Frame:Delete()
end

do
	local function Sort(a, b)
		if a.Remaining == b.Remaining then
			return a.UniqueID < b.UniqueID
		end
		return a.Remaining < b.Remaining
	end
	function Anchor:DoLayout()
		table.sort(self.Bars, Sort)
		
		for i, bar in self:IterateBars() do
			if not bar.Deleting then
				if bar.OldOrder then -- Do Animation				
				else
					bar.OldOrder = i
				end
				bar.frame:ClearAllPoints()
				local j = i - 1
				if i == 1 then
					bar.frame:SetPoint("BOTTOMLEFT", self.Frame, "TOPLEFT")
				else
					bar.frame:SetPoint("BOTTOMLEFT", self.Frame, "TOPLEFT", 0, j * self.db.Height + self.db.Spacing * j)
				end
			end
		end
	end
end

function Anchor:AddBar(bar)
	tinsert(self.Bars, bar)
	
	Counter = Counter + 1
	if Counter > 1000 then Counter = 0 end
	bar.UniqueID = Counter
	
	bar.db = self.db.Bar
	bar.Anchor = self
	bar.StartTime = bar.Remaining
	
	bar.frame:SetWidth(self.db.Width)
	bar.frame:SetHeight(self.db.Height)
	bar:ApplySettings()
	
	self.Frame:SetScript("OnUpdate", OnUpdate)
	
	self:DoLayout()
end

function Anchor:RemoveBar(bar)
	for i, b in self:IterateBars() do
		if b == bar then
			tremove(self.Bars, i)
		end
	end
	
	if #self.Bars == 0 then
		self.Frame:SetScript("OnUpdate", nil)
	end
	
	self:DoLayout()
end

function Anchor:IterateBars()
	return pairs(self.Bars)
end

function Anchor:Show()
	self.Frame:Show()
end

function Anchor:Hide()
	self.Frame:FadeOut()
end

function Anchor:SetWidth(width)
	if DEBUG then
		expect(width, "typeof", "number")
	end
	self.Frame:SetWidth(width)
end

OnUpdate = function(this, elapsed)
	local self = this.obj
	for i, bar in self:IterateBars() do
		bar.Remaining = bar.Remaining - elapsed		
		
		local minutes = RW.Time:Minutes(bar.Remaining)
		local seconds = RW.Time:Seconds(bar.Remaining)
		local t = RW.Time:Thousands(bar.Remaining) * 10		
		
		if not bar.Deleting then
			bar:SetValue(1 - bar.Remaining / bar.StartTime)
			
			if bar.Remaining >= 60 then
				bar.timetext:SetFormattedText("%.1d:%.2d", minutes, seconds)
			elseif bar.Remaining > 15 then
				bar.timetext:SetFormattedText("%.2d", seconds)
			else
				bar.timetext:SetFormattedText("%.1d.%.1d", seconds, t)
			end	
		
			if bar.Remaining < self.db.MinTime and self.db.Next ~= "None" then
				local NewAnchor = plugin.AnchorsByName[self.db.Next]
				if NewAnchor then
					local scale = NewAnchor.Frame:GetEffectiveScale()
					local x, y = NewAnchor.Frame:GetLeft() * scale, NewAnchor.Frame:GetBottom() * scale
					local scale2 = bar.frame:GetEffectiveScale()
					local x2, y2 = bar.frame:GetLeft() * scale2, bar.frame:GetBottom() * scale2
					local dx, dy = x - x2, y - y2
					
					local newBar = bar:Copy()
					bar.DoTranslateDelete = true
					bar.TranslateX = dx
					bar.TranslateY = dy
					bar:Delete()
					
					plugin.IDLookup[newBar.ID] = newBar
					
					NewAnchor:AddBar(newBar)
				end
				
			end
			
			if bar.Remaining < 0 then
				bar.timetext:SetFormattedText("%.1d", 0)
				bar:Delete()
			end
		end		
	end
end

OnDragStart = function(self)
	self:StartMoving()
end
OnDragStop = function(self)
	self:StopMovingOrSizing()
	
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	self.obj.db.AnchorFrame = "BlizzUIParent"
	self.obj.db.Point = "CENTER"
	self.obj.db.RelPoint = "CENTER"
	self.obj.db.x = x
	self.obj.db.y = y
	
	if plugin.OptionsActive then
		RW.Config:Toggle(true)
	end
end