local name, addon_table = ...

local _G = _G
local RW = _G.RW3
local expect = RW.Debug.expect
local DEBUG = RW.DEBUG

local plugin = RW:NewPlugin("Timers")
addon_table.Plugin = plugin
RW.Debug:AddGlobal("Timers", plugin)

local LSM = LibStub("LibSharedMedia-3.0")
local FA = LibStub("LibFrameAnchorRegistry-1.0")

local Constants = plugin.Constants
Constants:Register("NO_NEXT", 50)

local Anchors = {}
local AnchorsByName = {}
local db
local options

plugin.IDLookup = {}
plugin.Anchors = Anchors
plugin.AnchorsByName = AnchorsByName

function plugin:OnPluginInitialize()

	-- Register DB
	local defaults = {
		profile = {
			Enabled = true,
			
			-- Anchor
			DefaultAnchor = "Default",
			Anchor = {
				["**"] = {
					Point = "CENTER",
					AnchorFrame = "BlizzUIParent",
					RelPoint = "CENTER",					
					x = 0,
					y = 0,
					
					Width = 200,
					Height = 30,
					Spacing = 2,
					
					Deletable = true,
					
					Next = "None",
					
					Bar = {
						ShowIcon = true,
						IconSide = Constants.ICONSIDE_LEFT,
						
						BorderTexture = "",
						BorderWidth = 10,
						BorderColor = {1, 1, 1, 1},
						
						BackgroundColor = {0, 0, 0, 0.5},
						
						Inset = 1,
						
						StatusBarTexture = "",
						StatusbarColor = {1, 1, 1, 1},
						
						Font = "",
						FontSize = 12,
						FontFlags = "OUTLINE",
						
						FontShadowColor = {0, 0, 0, 1},
						FontShadowOffsetX = 0,
						FontShadowOffsetY = 0,			

						TimeAreaSize = 65,		
						
					},
				},
				{
					Name = "Default",
					Deletable = false,
					MinTime = 15,
					Next = "Default2",
				},
				{
					Name = "Default2",
					Deletable = false,
					MinTime = 0,
					Next = Constants.NO_NEXT,
				},
				{
					Name = "Default3",
					Deletable = false,
					MinTime = 0,
					Next = Constants.NO_NEXT,
				},
			}
		}
	}
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
		
	-- Set Enabled state
	self:SetEnabledState(db.Enabled)
end

function plugin:OnPluginEnable()	
	self:OnReset()
	
	RW.RegisterCallback(self, "Timer", "CallbackTimer")
	RW.RegisterCallback(self, "TimerSpell", "CallbackTimerSpell")
	
	RW.RegisterCallback(self, "TimerStop", "CallbackTimerStop")
end

function plugin:OnReset(event, ...)
	db = self.db.profile
	
	for i, anchor in pairs(Anchors) do
		anchor:Delete()
	end
	wipe(Anchors)
	wipe(AnchorsByName)
	for i, anchor in pairs(db.Anchor) do
		local a = self:CreateAnchor(anchor.Name)		
		a.db = anchor
		tinsert(Anchors, a)
		AnchorsByName[anchor.Name] = a
		
		for j, bar in a:IterateBars() do
			bar.db = anchor.Bar
		end
	end
	
	self:ApplySettings()
end

function plugin:ApplySettings()
	for i, a in pairs(Anchors) do
		local AnchorFrame = FA:GetAnchor(a.db.AnchorFrame or "BlizzUIParent")
		local scale = a.Frame:GetEffectiveScale() / UIParent:GetEffectiveScale()
		local x, y = a.db.x / scale, a.db.y / scale
		a.Frame:ClearAllPoints()
		a.Frame:SetPoint(a.db.Point, AnchorFrame, a.db.RelPoint, x, y)
		a:SetWidth(a.db.Width)
		
		a:ApplySettings()
	end
end

function plugin:ToggleConfigMode(event, toggle)
	if toggle then
		for i, anchor in pairs(Anchors) do
			anchor:Show()
		end
	else
		for i, anchor in pairs(Anchors) do
			anchor:Hide()
		end
	end
end

function plugin:OnConfigLoaded()
	RW.Config:AddPlugin("Timers", options)
end

--- Starts a timer with the given duration and text
-- @name Callback:Timer
-- @param ID Unique ID to identify this timer
-- @param Duration Duration of the timer in seconds
-- @param Text Text to display on the timer
-- @param _Anchor Optional param to control what anchor the timer should appear on
function plugin:CallbackTimer(event, ID, Duration, Text, _Anchor, ... )
	if DEBUG then
		expect(ID, "typeof", "string;number")
		expect(Duration, "typeof", "number")
		expect(Text, "typeof", "string;number")
	end
end

--- Start a timer for a specific spell
-- @name Callback:TimerSpell
-- @param ID Unique ID to identify this timer
-- @param Duration Duration of the timer in seconds
-- @param Spell The spell to take the icon and text from
-- @param _Anchor Optional param to control what anchor the timer should appear on
function plugin:CallbackTimerSpell(event, ID, Duration, Spell, _Anchor)
	if DEBUG then
		expect(ID, "typeof", "string;number")
		expect(Duration, "typeof", "number")
		expect(Spell, "typeof", "number")
	end
	
	local Anchor
	if _Anchor and AnchorsByName[_Anchor] then
		Anchor = AnchorsByName[_Anchor]
	else
		Anchor = AnchorsByName[db.DefaultAnchor]
	end
		
	local bar
	if self.IDLookup[ID] then
		bar = self.IDLookup[ID]
		if bar.Anchor ~= Anchor then
			bar:Delete()
			bar = nil
		end
	end
	if not bar then
		bar = self:CreateBar()
	end	
	
	bar.Remaining = Duration
	bar.ID = ID
	
	if not bar.Anchor then
		Anchor:AddBar(bar)
	else
		bar.Anchor:DoLayout()
	end
	
	self.IDLookup[ID] = bar
	local SpellName, _, Icon = GetSpellInfo(Spell)
	bar:SetName(SpellName)
	bar:SetIcon(Icon)
end

--- Stop a timer
-- @name Callback:TimerStop
-- @param ID The unique ID of the timer to stop
function plugin:CallbackTimerStop(event, ID)
	if DEBUG then
		expect(ID, "typeof", "string;number")		
	end
	
	if self.IDLookup[ID] then
		self.IDLookup[ID]:Delete()
		self.IDLookup[ID] = nil
	end
end

function GetAnchorList(Current)
	local list = {}
	for i, anchor in pairs(db.Anchor) do
		if (Current and anchor.Name ~= Current) or not Current then		
			tinsert(list, {
				widget = "CI-Dropdown-Item",
				name = anchor.Name,
				value = anchor.Name,
			})
		end
	end
	
	tinsert(list, {
				widget = "CI-Dropdown-Item",
				name = "None",
				value = Constants.NO_NEXT,
			})
	
	return list
end
local status = {}
plugin.OptionsActive = false
options = {
	name = "Timers",
	icon = RW.Utils:IconTexture("Spell_Shadow_Dispersion"),
	groups = {
		{
			name = "Anchors",
			children = {
				{
					widget = "CI-DropdownGroup",
					type = "group",
					layout = "Fill",
					fullwidth = true,
					fullheight = true,
					oncreated = function(widget) 
						plugin.OptionsActive = true
						widget:SetCallback("OnRelease", function()
							plugin.OptionsActive = false
						end)
					end,
					groups = function()
						local r = {}
						for i, v in pairs(db.Anchor) do							
							tinsert(r, {
								widget = "CI-Dropdown-Item",
								name = v.Name,
								value = i,								
								children = {
									{
										widget = "ScrollFrame", type = "group", layout = "EvenRow", statustable = status,
										db = function() return v end,
										callback = function() plugin:ApplySettings() end,
										children = {	
											{ widget = "CI-InlineGroup", name = "Position And Size", type = "group", layout = "EvenRow", fullwidth = true, template = "Group1",
												children = {
													{ widget = "CI-Dropdown", name = "Anchor", relwidth = 1, key = "AnchorFrame",
														items = function() return RW.Config:GetAnchorItems(AnchorsByName[v.Name].Frame) end,
														callback = function()
															v.RelPoint = "CENTER"
															v.Point = "CENTER"
															v.x = 0
															v.y = 0
															plugin:ApplySettings()
															if plugin.OptionsActive then
																RW.Config:Toggle(true)
															end
														end,
													},
													{ widget = "CI-GridSelect", name = "Point", relwidth = 0.5, type = "Anchor", columns = 3, height = 75, layout = "Relative" },
													{ widget = "CI-GridSelect", name = "Relative Point", key = "RelPoint", relwidth = 0.5, type = "Anchor", columns = 3, height = 75, layout = "Relative" },
													{ widget = "CI-Slider", name = "X", key = "x", min = -1500, max = 1500, step = 0.1, relwidth = 0.5 },
													{ widget = "CI-Slider", name = "Y", key = "y", min = -1500, max = 1500, step = 0.1, relwidth = 0.5 },
													{ widget = "CI-Slider", name = "Width", min = 20, max = 1000, step = 0.1, relwidth = 0.5 },
													{ widget = "CI-Slider", name = "Height", min = 10, max = 250, step = 0.1, relwidth = 0.5 },
												},
											},
											{ widget = "CI-InlineGroup", name = "Other", type = "group", layout = "EvenRow", fullwidth = true, template = "Group1",
												db = function() return v.Bar end,
												children = {
													{ widget = "CI-Slider", name = "Border Width", key = "BorderWidth", min = 0, max = 50, step = 0.1, relwidth = 0.5 },
													{ widget = "CI-Slider", name = "Border Inset", key = "Inset", min = 0, max = 50, step = 0.1, relwidth = 0.5 },	
													{ widget = "CI-ColorSelect", name = "Background Color", key = "BackgroundColor", relwidth = 0.5 },
													{ widget = "CI-NewLine" },
													{ widget = "CI-ColorSelect", name = "Border Color", key = "BorderColor", relwidth = 0.5 },
													{ widget = "CI-ColorSelect", name = "Bar Color", key = "StatusbarColor", relwidth = 0.5 },
													{ 												
														widget = "CI-Dropdown", name = "Border", key = "BorderTexture", relwidth = 0.5,
														items = function() return RW.Media:GetBorderItems() end,
													},
													{
														widget = "CI-Dropdown", name = "StatusBar Texture", key = "StatusBarTexture", relwidth = 0.5,
														items = function() return RW.Media:GetStatusBarItems() end,
													},
													{ widget = "CI-InlineGroup", name = "Font", fullwidth = true, type = "group", layout = "EvenRow", template = "Group2",
														children = {
															{ widget = "CI-Dropdown", name = "Font", relwidth = 0.5,
																items = function() return RW.Media:GetFontItems() end
															},
															{ widget = "CI-Dropdown", name = "Outline", key = "FontFlags", relwidth = 0.5,
																items = function() return RW.Media:GetOutlineItems() end,
															},
															{ widget = "CI-Slider", name = "Font Size", key = "FontSize", min = 1, max = 50, step = 1, relwidth = 0.5, },
															{ widget = "CI-ColorSelect", name = "Shadow Color", key = "FontShadowColor", relwidth = 0.5 },
															{ widget = "CI-Slider", name = "Font Shadow Offset X", key = "FontShadowOffsetX", min = -50, max = 50, step = .1, relwidth = 0.5, },
															{ widget = "CI-Slider", name = "Font Shadow Offset Y", key = "FontShadowOffsetY", min = -50, max = 50, step = .1, relwidth = 0.5, },
														},
													},
													{ widget = "CI-Slider", name = "Time area size", key = "TimeAreaSize", min = 0, max = 200, step = 1, relwidth = 0.5 },													
												},
											},
											{ widget = "CI-Slider", name = "Spacing", min = -20, max = 20, step = 0.1, relwidth = 0.5 },
											{ widget = "CI-NewLine" },
											{ widget = "CI-Dropdown", text = "Next", key = "Next", items = function() return GetAnchorList(v.Name) end },											
										}
									}
								}
							})
						end
						return r
					end,
				}
			},
		}
	},
	children = {		
		{
			widget = "SimpleGroup",
			layout = "EvenRow",
			db = function() return db end,
			callback = function(...)
				plugin:ApplySettings()
			end,
			type = "group",
			children = {
				{ widget = "CI-Button", text = "Start", callback = function()
					RW.Callbacks:Fire("TimerSpell", "Test", 20, 62323)
				end},
				{ widget = "CI-Button", text = "Stop", callback = function()
					RW.Callbacks:Fire("TimerStop", "Test")
				end},
				{ widget = "CI-NewLine" },
				{ widget = "CI-Button", text = "Start", callback = function()
					RW.Callbacks:Fire("TimerSpell", "Test2", 60, 42587)
				end},
				{ widget = "CI-Button", text = "Stop", callback = function()
					RW.Callbacks:Fire("TimerStop", "Test2")
				end},
				{ widget = "CI-NewLine" },
				{ widget = "CI-Button", text = "Start", callback = function()
					RW.Callbacks:Fire("TimerSpell", "Test3", 870.1, 2548)
				end},
				{ widget = "CI-Button", text = "Stop", callback = function()
					RW.Callbacks:Fire("TimerStop", "Test3")
				end},
			}
		}
	}
}